--*********************************************************
--* FILE  : PD_ASIC_ALG.VHD
--* Author: Jack Fried
--*
--* Last Modified: 6/7/2010
--*  
--* ADC ltc2366
--*		 		               
--*
--*
--*
--*
--*
--*
--*********************************************************

LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;


--  Entity Declaration

ENTITY ADC_AD7276_OS IS

	PORT
	(
		clk      	: IN STD_LOGIC;						-- clock
		rst     		: IN STD_LOGIC;						-- state machine reset
		start   		: IN STD_LOGIC;						-- state machine reset
		ADC_SDO     : IN STD_LOGIC;						-- state machine reset
		OVER_SAMP	: IN STD_LOGIC_VECTOR(1 downto 0);						-- state machine reset
		ADC_SCK		: OUT STD_LOGIC;					-- state machine reset
		ADC_CS		: OUT STD_LOGIC;					-- state machine reset
		DATA_OUT		: OUT STD_LOGIC_VECTOR(15 downto 0);-- state machine reset
		RDY			: OUT STD_LOGIC						-- state machine reset
	);

END ADC_AD7276_OS;


ARCHITECTURE behavior OF ADC_AD7276_OS IS



  type state_type is (
		S_IDLE,
		S_start_ADC,
		S_ADC_READ,
		S_end_adc_read, 
		S_NEXT_SAMPLE,
		S_DIV_SAMPLE,
		S_done
		);

  signal state: state_type;
  signal CLK_CNT    : STD_LOGIC_VECTOR(7 downto 0);
  signal DLY_CNT    : STD_LOGIC_VECTOR(7 downto 0);
  signal clr_cnt	  : STD_LOGIC;
  signal OS_cnt	  : STD_LOGIC_VECTOR(3 downto 0);
  signal Data 	  	  : STD_LOGIC_VECTOR(15 downto 0);
  signal Data_sum	  : STD_LOGIC_VECTOR(15 downto 0);
  signal s_start	  : STD_LOGIC;

begin


 process(clk)  
  begin
		if (clk'event  AND  clk = '1') then
			s_start	<= start;
		end if;
end process;		



	ADC_SCK		<=	clk;
  process(clk,rst) 
  begin

     if (rst = '1') then
		ADC_CS		<= '1';
		RDY 			<= '0';
		CLK_CNT  	<= X"00";
		DATA_OUT		<= X"0000";
		DATA			<= X"0000";
		OS_cnt		<= x"0";
		DLY_CNT		<=	x"00";
     elsif (clk'event AND clk = '1') then
     CASE state IS
     when S_IDLE =>	
			ADC_CS		<= '1';
			CLK_CNT  	<= X"00";
			OS_cnt		<= x"0";
			Data_sum		<= x"0000";
			DLY_CNT		<=	x"00";
			if (s_start = '1')  then
				state 			<= S_ADC_READ;	
				RDY 			<= '0';
				DATA			<= X"0000";
				DATA_OUT		<= X"0000";
				Data_sum		<= x"0000";
				ADC_CS		<= '0';
			else
  	    	  state 		<= S_IDLE;
            end if;		
     when S_ADC_READ =>	
			CLK_CNT	 	<= CLK_CNT + 1;
			Data(0) 		<= ADC_SDO;			
			for i in 0 to 13 loop 
				Data(i+1) <= Data(i); 
			end loop;
			if (CLK_CNT >= 13) then
				state 		<= S_end_adc_read;
			else
				state 	<= S_ADC_READ;
			end if;

	 when S_end_adc_read =>
			ADC_CS		<= '1';
			CLK_CNT  	<= X"00";
			OS_cnt		<= OS_cnt + 1;
			DLY_CNT		<=	x"00";
			Data_sum	<= Data_sum + DATA(11 downto 0);
			if(OVER_SAMP = 0) then
				state       <= S_DIV_SAMPLE;		
			elsif((OVER_SAMP = 1) and  (OS_cnt = x"3"))then
				state       <= S_DIV_SAMPLE;	
			elsif((OVER_SAMP = 2) and  (OS_cnt = x"7"))then
				state       <= S_DIV_SAMPLE;
			elsif((OVER_SAMP = 3) and  (OS_cnt >= x"f"))then
				state       <= S_DIV_SAMPLE;
			else
				state       <= S_NEXT_SAMPLE;	
			end if;
	 when S_NEXT_SAMPLE =>	
			DLY_CNT	<= DLY_CNT + 1;
			ADC_CS		<= '1';
			CLK_CNT  	<= X"00";
			if(DLY_CNT >= 8) then
				state 		<= S_ADC_READ;	
				ADC_CS		<= '0';
			end if;
	 when S_DIV_SAMPLE =>	
			ADC_CS		<= '1';
			CLK_CNT  	<= X"00";
			if(OVER_SAMP = 0) then 			--- NO OVER SAMPLE
				DATA_OUT		<= b"00" & DATA(11 downto 0) & b"00";	
			elsif(OVER_SAMP = 1)then		--- 4 x over sample
				DATA_OUT		<= Data_sum(15 downto 0);	
			elsif(OVER_SAMP = 2)then		--- 8 x over sample
				DATA_OUT		<= '0' & Data_sum(15 downto 1);	
			elsif(OVER_SAMP = 3)then		-- 16 x over sample
				DATA_OUT		<= b"00" & Data_sum(15 downto 2);	
			end if;	
			state       <=	S_done;
	 when S_done =>
			ADC_CS		<= '1';
			CLK_CNT  	<= X"00";
			RDY 			<= '1';
 			if (s_start = '1')  then
				state <= S_done;	
			else
				state <= S_idle;	
			end if;	 
	 when others =>	
			state <= S_idle;		
      end case;  
	
		
     end if;
end process;

	
END behavior;
