--*********************************************************
--* FILE  : VMM_READOUT.VHD
--* Author: Jack Fried
--*
--* Last Modified: 3/30/2012
--*  
--* VMM_READOUT
--*		 		               
--*
--*
--*
--*
--*
--*
--*********************************************************

LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;


--  Entity Declaration

ENTITY VMM2_READOUT IS

	PORT
	(
		clk      		: IN STD_LOGIC;						-- clock 50Mhz
		rst     			: IN STD_LOGIC;						-- state machine reset
		ACQ_enable 		: IN STD_LOGIC;						-- state start
		ART_EN			: IN STD_LOGIC;		
		TST_PULSE		: IN STD_LOGIC;
		CKBC_PERIOD		: IN STD_LOGIC_VECTOR(15 downto 0);
		ADC_INT_EN		: IN STD_LOGIC; 						-- internal ADC ENABLE
		SW_EN				: IN STD_LOGIC; -- enable usb cnt of IO
		SW_vmm2_TKI		: IN STD_LOGIC; 
		SW_vmm2_DI		: IN STD_LOGIC; 
		SW_vmm2_CKTK	: IN STD_LOGIC; 
		SW_vmm2_ENA		: IN STD_LOGIC; 
		SW_vmm2_CKBC	: IN STD_LOGIC; 
		SW_vmm2_CKART	: IN STD_LOGIC; 
		SW_vmm2_CKDT	: IN STD_LOGIC; 
		SW_VMM2_WEN		: IN STD_LOGIC; 
		SW_VMM2_CKTP	: IN STD_LOGIC; 

	
		VMM2_TKI			: OUT STD_LOGIC;	
		VMM2_DI			: OUT STD_LOGIC;			
		VMM2_CKTK		: OUT STD_LOGIC;	
		VMM2_ENA			: OUT STD_LOGIC;	
		VMM2_CKBC		: OUT STD_LOGIC;
		VMM2_CKART		: OUT STD_LOGIC;
		VMM2_CKDT		: OUT STD_LOGIC;	
		VMM2_WEN			: OUT STD_LOGIC;	
		VMM2_CKTP		: OUT STD_LOGIC;	
	
		VMM2_ART			: IN STD_LOGIC;	
		VMM2_SETT		: IN STD_LOGIC;	
		VMM2_DATA1		: IN STD_LOGIC;	
		VMM2_DATA0		: IN STD_LOGIC;	
		VMM2_TKO			: IN STD_LOGIC;	
		ADDR_OUT			: OUT STD_LOGIC_VECTOR(7 downto 0);
		USB_DATA			: OUT STD_LOGIC_VECTOR(15 downto 0);
		USB_LATCH		: OUT STD_LOGIC
	
	);

END VMM2_READOUT;


ARCHITECTURE behavior OF VMM2_READOUT IS




	
	type state_type is (
						S_IDLE,
						S_acq_WAITING,
						S_acq_START,	
						S_acq_DATA_CKDT_H,
						S_acq_DATA_CKDT_L,
						S_acq_DATA_ADDR,
						S_acq_NEXT_CHN_START,
						S_acq_WAITING_adc,
						S_acq_DATA_CKDT_H_ADC,
						S_acq_DATA_CKDT_L_ADC,
						S_acq_DATA_ADC, 
						S_STORE_DATA_ADC);

	SIGNAL state			: state_type;
	
	SIGNAL CNT 	   		: STD_LOGIC_VECTOR(15 downto 0);
	SIGNAL COUNT	  		: STD_LOGIC_VECTOR(15 downto 0);
	SIGNAL clk_cnt	  		: STD_LOGIC_VECTOR(15 downto 0);
	SIGNAL CHN_ADDR	 	: STD_LOGIC_VECTOR(7 downto 0);
	SIGNAL CHN_ADDR_S	 	: STD_LOGIC_VECTOR(7 downto 0);
	SIGNAL ADDR_BIT		: INTEGER range 0 to 63;
	SIGNAL ART_CNT			: INTEGER range 0 to 31;
	SIGNAL TKI				: STD_LOGIC;	
	SIGNAL CKTK				: STD_LOGIC;	
	SIGNAL ENA				: STD_LOGIC;	
	SIGNAL CKBC				: STD_LOGIC;	
	SIGNAL CKDT				: STD_LOGIC;
	SIGNAL ART				: STD_LOGIC;
	SIGNAL DATA0			: STD_LOGIC;
	SIGNAL DATA1			: STD_LOGIC;
	SIGNAL TKO				: STD_LOGIC;

	SIGNAL VMM2_DATA_i 	: STD_LOGIC_VECTOR(37 downto 0);
	SIGNAL VMM2_DATA_S 	: STD_LOGIC_VECTOR(37 downto 0);
	SIGNAL USB_LATCH_DLY	: STD_LOGIC;
	SIGNAL LATCH_data		: STD_LOGIC;
	SIGNAL DATA_SEL		: STD_LOGIC_VECTOR(7 downto 0);
	SIGNAL USB_DATA_dly  : STD_LOGIC_VECTOR(15 downto 0);
	
	
	SIGNAL vmm2_d_add  	: STD_LOGIC_VECTOR(5 downto 0);		
	SIGNAL vmm2_d_ts     : STD_LOGIC_VECTOR(11 downto 0);
	SIGNAL vmm2_d_ampl  	: STD_LOGIC_VECTOR(9 downto 0);
	SIGNAL vmm2_d_time   : STD_LOGIC_VECTOR(7 downto 0);
	SIGNAL vmm2_d_flag   : STD_LOGIC_VECTOR(1 downto 0);	
	SIGNAL art_addr	   : STD_LOGIC_VECTOR(7 downto 0);		
	SIGNAL art_rst		   : STD_LOGIC;
begin



	
		VMM2_TKI			<=	TKI	WHEN (SW_EN = '0') ELSE SW_vmm2_TKI;
		VMM2_DI			<= SW_vmm2_DI;
		VMM2_CKTK		<=	CKTK	WHEN (SW_EN = '0') ELSE SW_vmm2_CKTK;
		VMM2_ENA			<=	ENA	WHEN (SW_EN = '0') ELSE SW_vmm2_ENA	;
		VMM2_CKBC		<=	CKBC	WHEN (SW_EN = '0') ELSE SW_vmm2_CKBC;
		VMM2_CKART		<=	clk   WHEN ((SW_EN = '0') and (ART_EN = '1')) ELSE SW_vmm2_CKART;
		VMM2_CKDT		<=	CKDT	WHEN (SW_EN = '0') ELSE SW_vmm2_CKDT;
		VMM2_WEN			<=	SW_VMM2_WEN;
		VMM2_CKTP		<=	TST_PULSE WHEN (SW_EN = '0') ELSE SW_VMM2_CKTP;
		
		
			
	vmm2_d_add    	<=  VMM2_DATA_S(7 downto 2);
	vmm2_d_ts		<=  VMM2_DATA_S(37 downto 26);
	vmm2_d_ampl    <=  VMM2_DATA_S(17 downto 8);	
	vmm2_d_time    <=  VMM2_DATA_S(25 downto 18);	
	vmm2_d_flag  	<=	 VMM2_DATA_S(1  downto 0);	
		
		


		USB_DATA_dly	<= x"ff" & art_addr										when (DATA_SEL = x"00") else
								b"00" & vmm2_d_add & vmm2_d_time		   		when (DATA_SEL = x"01") else
								b"00" & vmm2_d_flag &  b"00" & vmm2_d_ampl 	when (DATA_SEL = x"02") else
								x"0"  & vmm2_d_ts										when (DATA_SEL = x"03") else
								x"0000";

--- SYNC TO  50Mhz CLOCK ----	
	
 process(clk)  
  begin
	if (clk'event  AND  clk = '1') then
			DATA0		<= VMM2_DATA0;
			DATA1		<=	VMM2_DATA1;
			TKO		<=	VMM2_TKO;
			ADDR_OUT	<= CHN_ADDR_S;
			ART		<= VMM2_ART;
	  end if;
end process;		
		

		
		CKBC 	<= clk_cnt(0)   when (CKBC_PERIOD = x"0000") else
					clk_cnt(1)   when (CKBC_PERIOD = x"0001") else
					clk_cnt(2)   when (CKBC_PERIOD = x"0002") else
					clk_cnt(3)   when (CKBC_PERIOD = x"0003") else
					clk_cnt(4)   when (CKBC_PERIOD = x"0004") else
					clk_cnt(5)   when (CKBC_PERIOD = x"0005") else
					clk_cnt(6)   when (CKBC_PERIOD = x"0006") else
					clk_cnt(7)   when (CKBC_PERIOD = x"0007") else
					clk_cnt(8)   when (CKBC_PERIOD = x"0008") else	
					clk_cnt(9)   when (CKBC_PERIOD = x"0009") else
					clk_cnt(10)   when (CKBC_PERIOD = x"000a") else
					clk_cnt(11)   when (CKBC_PERIOD = x"000b") else
					clk_cnt(12)   when (CKBC_PERIOD = x"000c") else	
					clk_cnt(13)   when (CKBC_PERIOD = x"000d") else						
					clk_cnt(14)   when (CKBC_PERIOD = x"000e") else						
					clk_cnt(15)   when (CKBC_PERIOD = x"000f") else						
					'0';

		
		
 process(clk)  
  begin
	  if ((rst = '1') or (SW_EN = '1')) then
	  		DATA_SEL			<= x"00";
			USB_LATCH		<= '0';
			clk_cnt			<= x"0000";
			art_rst			<= '0';
		elsif (clk'event AND clk = '1') then
			clk_cnt <= clk_cnt +1;
			USB_LATCH		<= '0';
			USB_DATA			<= USB_DATA_dly;
			art_rst			<= '0';
			if( LATCH_data	= '1') and (DATA_SEL = x"00" )then
				USB_LATCH	<= '1';
				art_rst		<= '1';
				DATA_SEL 	<= x"01";
			end if;
			if(DATA_SEL >= x"01" )then
				USB_LATCH	<= '1';
				DATA_SEL 	<= DATA_SEL + 1;				
			end if;
			if(DATA_SEL = x"03" )then
				DATA_SEL		<= x"00";
			end if;			
		end if;
end process;		
			

	 process(clk)  
  begin
	  if  ((rst = '1') or (art_rst = '1')) then
			art_addr	<= x"ff";
			ART_CNT	<= 0;
	  elsif (clk'event AND clk = '1') then		
	  
		if (ART = '1') and (ART_CNT = 0) then
			ART_CNT	<= 1;
			art_addr	<= x"00";
		end if;
		if(ART_CNT /= 0) then
			ART_CNT	<= ART_CNT + 1;
		end if;
		if(ART_CNT > 1) then
			art_addr(ART_CNT - 2) <= ART;
		end if;
		if(ART_CNT >= 7) then
			ART_CNT <= 0;
		end if;
	  end if;
end process;		
			
		
		
		
-- start main read out

  process(clk,rst,SW_EN) 
  begin

     if ((rst = '1') or (SW_EN = '1')) then
		TKI			<= '0';
		CKTK			<= '0';
		ENA			<= '0';
		CKDT			<= '0';
		COUNT			<= X"0000";
		CNT			<= X"0000";
		CHN_ADDR		<= X"00";
		CHN_ADDR_S	<= X"00";
		ADDR_BIT		<=	0;
		state 		<= S_IDLE;
		VMM2_DATA_i	<= (others => '0');
		VMM2_DATA_s	<= (others => '0');		
		LATCH_data	<= '0';
	elsif (clk'event AND clk = '1') then
	
		CASE state IS
		when S_IDLE =>	
				VMM2_DATA_i	<= (others => '0');
				VMM2_DATA_s	<= (others => '0');		
				TKI			<= '0';
				CKTK			<= '0';
				ENA			<= '0';
				CKDT			<= '0';
				CHN_ADDR		<= X"00";
				CHN_ADDR_S	<= X"00";
				ADDR_BIT		<=	0;			
				COUNT			<= X"0000";
				CNT			<= X"0000";
				LATCH_data	<= '0';
				if (ACQ_enable  = '1')  then 
					if( ADC_INT_EN = '0') then
						ENA			<= '1';
						state 		<= S_acq_WAITING;
					else
						ENA			<= '1';
						state 		<= S_acq_WAITING_adc;
					end if;					
				end if;
		when S_acq_WAITING =>		
				ENA		<= '1';
				TKI		<= '0';
				CKTK		<= '0';
				CKDT		<= '0';
				COUNT		<= X"0000";
				CNT		<= X"0000";
				CHN_ADDR	<= X"00";
				ADDR_BIT	<=	0;			
				LATCH_data	<= '0';			
				IF( DATA0 = '1') THEN 
					state 		<= S_acq_START;
				END IF;
				IF(ACQ_enable  = '0')  THEN 
					state 	<= S_IDLE;
				END IF;
					
		when S_acq_START =>	
				COUNT		<= COUNT + 1;
				ADDR_BIT	<=	0;	
				IF(COUNT = 5) THEN
					ENA		<= '0';
				ELSIF (COUNT = 10) THEN
					TKI		<= '1';
				ELSIF (COUNT = 15) THEN
					CKTK		<= '1';
				ELSIF (COUNT >= 1000) THEN   --17
					COUNT			<= X"0000";
					state 		<= S_acq_DATA_CKDT_H;
					CHN_ADDR(ADDR_BIT) <= VMM2_DATA0;
					
				END IF;
		WHEN S_acq_DATA_CKDT_H =>
				CKDT		<= '1';
				ADDR_BIT	<= ADDR_BIT + 1;
				state 	<= S_acq_DATA_CKDT_L;
				CNT <= X"0000";
		WHEN S_acq_DATA_CKDT_L =>
				CKDT		<= '0';
				CNT		<= CNT + 1;
				IF( CNT >= 2) THEN 
					state 	<= S_acq_DATA_ADDR;
				END IF;
		WHEN S_acq_DATA_ADDR =>
				IF( ADDR_BIT >= 8) THEN
					CKTK		<= '0';
					COUNT		<= X"0000";
					state 	<= S_acq_NEXT_CHN_START;
				ELSE
					CHN_ADDR(ADDR_BIT) <= VMM2_DATA0;
					state 	<= S_acq_DATA_CKDT_H;
				END IF;
		WHEN S_acq_NEXT_CHN_START =>
				ADDR_BIT 	<= 0;
				CHN_ADDR_S	<= CHN_ADDR;	
				COUNT			<= COUNT + 1;
				LATCH_data	<= '0';
				IF(COUNT = 1) THEN
					LATCH_data	<= '0';
					TKI			<= '0';		
				ELSIF (COUNT = 5) THEN
					CKTK		<= '1';
				ELSIF (COUNT = 10) THEN
					IF( DATA0 = '1') THEN 
						CHN_ADDR(ADDR_BIT) <= DATA0;
						COUNT					 <= X"0000";
						state 				 <= S_acq_DATA_CKDT_H;
					END IF;
				ELSIF (COUNT = 12) THEN
					CKTK		<= '0';
				ELSIF (COUNT = 14) THEN
					CKTK		<= '1';
				ELSIF (COUNT = 16) THEN
					CKTK		<= '0';
				ELSIF (COUNT = 18) THEN
					CKTK		<= '1';			
				ELSIF (COUNT = 20) THEN
					CKTK		<= '0';
				ELSIF (COUNT = 22) THEN
					CKTK		<= '1';				
				ELSIF (COUNT = 24) THEN
					CKTK		<= '0';
				ELSIF (COUNT = 26) THEN
					CKTK		<= '1';				
				ELSIF (COUNT >= 28) THEN
					CKTK		<= '0';				
					state 	<= S_acq_WAITING;
				END IF;		
				IF(ACQ_enable  = '0')  THEN 
					state 	<= S_IDLE;
				END IF;	
	----------------------------------------------------- ADC READOUT			
				

		WHEN	S_acq_WAITING_adc =>		
				CNT		<=  CNT + 1;
				ENA		<= '1';
				TKI		<= '0';
				CKTK		<= '0';
				CKDT		<= '0';
				COUNT		<= X"0000";
				CHN_ADDR	<= X"00";
				ADDR_BIT	<=	0;			
				LATCH_data	<= '0';	
				VMM2_DATA_i	<= (others => '0');	
				IF( CNT >= 8) then
					CKTK		<= '1';
				end if;
				IF( CNT >= 11) then
					CNT		<=  x"0000";
				end if;				
				IF( DATA0 = '1') THEN 
					CKTK		<= '0';
					CNT		<=  x"0000";
					state 	<= S_acq_DATA_CKDT_H_ADC;
					VMM2_DATA_i   <=  VMM2_DATA1 & VMM2_DATA0 &	VMM2_DATA_i(37 downto 2);  
				END IF;
				IF(ACQ_enable  = '0')  THEN 
					CKTK		<= '0';
					state 	<= S_IDLE;
				END IF;
				
		WHEN S_acq_DATA_CKDT_H_ADC =>
				CKDT		<= '1';
				CNT		<= CNT + 1;
				IF( CNT >= 2) THEN 
					CNT		<=  x"0000";	
					ADDR_BIT	<= ADDR_BIT + 2;
					state 	<= S_acq_DATA_CKDT_L_ADC;
				END IF;				
				
				
				
		WHEN S_acq_DATA_CKDT_L_ADC =>
				CKDT		<= '0';
				CNT		<= CNT + 1;
				IF( CNT >= 2) THEN 
					CNT		<=  x"0000";	
					state 	<= S_acq_DATA_ADC;
				END IF;
		WHEN S_acq_DATA_ADC =>
				IF( ADDR_BIT >= 38) THEN
					CKTK		<= '0';
					CNT		<=  x"0000";	
					state 	<= S_STORE_DATA_ADC;
					VMM2_DATA_S	<= VMM2_DATA_i;
				ELSE
					VMM2_DATA_i    <=  VMM2_DATA1 & VMM2_DATA0 &	VMM2_DATA_i(37 downto 2);  
					state 			<= S_acq_DATA_CKDT_H_ADC;
					CNT				<=  x"0000";
				END IF;		
		WHEN S_STORE_DATA_ADC =>
				LATCH_data	<= '1';
				CNT			<=  x"0000";
				state 		<= S_acq_WAITING_adc;
		when others =>	
			state <= S_idle;		
      end case;  
     end if;
end process;
	
	
	
END behavior;