--*********************************************************
--* FILE  : VMM_READOUT.VHD
--* Author: Jack Fried
--*
--* Last Modified: 3/30/2012
--*  
--* VMM_READOUT
--*		 		               
--*
--*
--*
--*
--*
--*
--*********************************************************

LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;


--  Entity Declaration

ENTITY VMM_READOUT IS

	PORT
	(
		clk      		: IN STD_LOGIC;						-- clock 50Mhz
		ADC_CLK  		: IN STD_LOGIC;						-- clock 48Mhz
		SPI_CLK			: IN STD_LOGIC;						-- clock 1Mhz
		rst     			: IN STD_LOGIC;						-- state machine reset
		ACQ_enable 		: IN STD_LOGIC;						-- state start
		usb_ADC_TST 	: IN STD_LOGIC_VECTOR(1 downto 0);	
		ADC_TST 			: IN STD_LOGIC;												-- ADC_TEST STROBE
		Trigger			: IN STD_LOGIC;
		ASIC_RESET		: IN STD_LOGIC;
		trig_cnt_rst	: IN STD_LOGIC;
		trig_timeout	: IN STD_LOGIC_VECTOR(15 downto 0);						-- time form pulse or flag till enable drops
		M_trigger		: IN STD_LOGIC;												-- mode trigger to enable 
		M_PUL_TO_EN 	: IN STD_LOGIC;												-- mode flag or pulse to enable 
		M_PUL_EDG_SEL	: IN STD_LOGIC;												-- mode pulse edge select 
		ENABLE_TIME		: IN STD_LOGIC_VECTOR(15 downto 0);						-- time form pulse or flag till enable drops
		TAC_STOP_TIME	: IN STD_LOGIC_VECTOR(15 downto 0);						-- time form flag to TAC STOP
		Start_DLY		: IN STD_LOGIC_VECTOR(15 downto 0);						-- time form flag to TAC STOP		
		IO_EN				: IN STD_LOGIC; -- enable usb cnt of IO
		IO_vmm_ck		: IN STD_LOGIC; -- reg1_p(0);
		IO_vmm_wen		: IN STD_LOGIC; -- reg1_p(1);
		IO_vmm_ena		: IN STD_LOGIC; -- reg1_p(2);
		IO_vmm_tkiA		: IN STD_LOGIC; -- reg1_p(3);
		IO_vmm_diA		: IN STD_LOGIC; -- reg1_p(5);
		IO_vmm_stp		: IN STD_LOGIC; -- reg1_p(8);
		IO_VMM_SCK		: IN STD_LOGIC; -- reg1_p(9);
		IO_VMM_CSB		: IN STD_LOGIC; -- reg1_p(10);

		ADC_Pdo_A		: IN STD_LOGIC;
		ADC_Tdo_A		: IN STD_LOGIC;
		ADC_SCK			: OUT STD_LOGIC; -- reg1_p(9);
		ADC_CSB			: OUT STD_LOGIC; -- reg1_p(10);	
		ADC_start_dly	: IN STD_LOGIC_VECTOR(15 downto 0);	 -- start delay for ADC wait for risetime of analog signal
		ADC_OVSAMP		: IN STD_LOGIC_VECTOR(1 downto 0);						-- time form flag to TAC STOP		
		vmm_A				: IN STD_LOGIC_VECTOR(5 downto 0);
		vmm_flagA		: IN STD_LOGIC;	
		vmm_ck			: OUT STD_LOGIC; -- reg1_p(0);
		vmm_wen			: OUT STD_LOGIC; -- reg1_p(1);
		vmm_ena			: OUT STD_LOGIC; -- reg1_p(2);
		vmm_tkiA			: OUT STD_LOGIC; -- reg1_p(3);
		vmm_diA			: OUT STD_LOGIC; -- reg1_p(5);
		vmm_stp			: OUT STD_LOGIC; -- reg1_p(8);
		spi_START		: IN STD_LOGIC;	
		spi_address 	: OUT STD_LOGIC_VECTOR(7 downto 0); 		
		spi_data			: IN  STD_LOGIC_VECTOR(31 downto 0);
		ADC_1_DAT		: OUT STD_LOGIC_VECTOR(15 downto 0);	
		ADC_2_DAT		: OUT STD_LOGIC_VECTOR(15 downto 0);	
		USB_wfull		: IN  STD_LOGIC;			
		USB_HS_CLK		: OUT STD_LOGIC;		
		USB_wr_req		: OUT STD_LOGIC;		
		USB_DATA			: OUT STD_LOGIC_VECTOR(15 downto 0);
		DATA_MODE		: IN STD_LOGIC_VECTOR(1 downto 0);		
		TEST_EN			: IN STD_LOGIC;		
		srs_header		: IN  STD_LOGIC_VECTOR(31 downto 0);
		hs_ffdata 		: OUT STD_LOGIC_VECTOR(7 downto 0);					
		hs_ff_PKT_END	: OUT STD_LOGIC;						
		hs_ff_wrreq 	: OUT STD_LOGIC;					
		hs_ff_wr_full  : IN STD_LOGIC;
		busy				: OUT STD_LOGIC				
	);

END VMM_READOUT;


ARCHITECTURE behavior OF VMM_READOUT IS




component ADC_AD7276_OS
	PORT
	(
		clk      	: IN STD_LOGIC;							-- clock
		rst     		: IN STD_LOGIC;							-- state machine reset
		start   		: IN STD_LOGIC;							-- Start ADC sampling
		ADC_SDO     : IN STD_LOGIC;							-- state machine reset
		OVER_SAMP	: IN STD_LOGIC_VECTOR(1 downto 0);	-- # of samples  00 = 1   01 = 4  10 = 8  11 = 16
		ADC_SCK		: OUT STD_LOGIC;							-- ADC serial clock
		ADC_CS		: OUT STD_LOGIC;							--	ADC CHIP SELECT
		DATA_OUT		: OUT STD_LOGIC_VECTOR(15 downto 0);-- ADC OUTPUT 
		RDY			: OUT STD_LOGIC							-- ADC SAMPLING RDY
	);
end component;
	

	
component VMM_SPI 
	PORT
	(
		rst 	   		: IN STD_LOGIC;
		clk_1Mhz   		: IN STD_LOGIC;						
		spi_START		: IN STD_LOGIC;	
		spi_address 	: OUT STD_LOGIC_VECTOR(7 downto 0); 		
		spi_data			: IN  STD_LOGIC_VECTOR(31 downto 0);
		spi_enabled		: OUT STD_LOGIC;		
		vmm_ck			: OUT STD_LOGIC; 
		vmm_wen			: OUT STD_LOGIC; 
		vmm_ena			: OUT STD_LOGIC; 
		vmm_diA			: OUT STD_LOGIC 
	);
end component;
	
	
component lx9_test 
	PORT
	(
		clk      		: IN STD_LOGIC;					
		rst     			: IN STD_LOGIC;						
		start   			: IN STD_LOGIC;						
		speed				: IN STD_LOGIC_VECTOR(15 downto 0);
		PKT_DLY			: IN STD_LOGIC_VECTOR(31 downto 0);
		hs_ffdata 		: OUT STD_LOGIC_VECTOR(7 downto 0);					
		hs_ff_PKT_END	: OUT STD_LOGIC;						
		hs_ff_wrreq 	: OUT STD_LOGIC;					
		hs_ff_wr_full  : IN STD_LOGIC			
		
	);
end component;
	
	
	

component ADC_UDP_TX 

	PORT
	(
		clk      			: IN STD_LOGIC;						-- clock 50Mhz
		rst     				: IN STD_LOGIC;						-- state machine reset
		start   				: IN STD_LOGIC;			
		srs_header			: IN  STD_LOGIC_VECTOR(31 downto 0);
		ADC1_DATA			: IN STD_LOGIC_VECTOR(15 downto 0);
		ADC2_DATA			: IN STD_LOGIC_VECTOR(15 downto 0);
		ADC_RDY				: IN STD_LOGIC;
		ADC_START_UDP		: OUT STD_LOGIC;
		
		hs_ADC_wrreq		: OUT STD_LOGIC;
		hs_ADC_pktend		: OUT STD_LOGIC;
		hs_ADC_data			: OUT STD_LOGIC_VECTOR(7 downto 0);
		hs_ff_wr_full  	: IN STD_LOGIC			
	);
end component;
	
	
	
	
	type state_type is (
					S_IDLE,
					S_delay_start,
					S_acq_reset_ENA,
					S_fast_reset_ENA,					
					S_wait_trigger,	
					S_wait_for_flag_or_tpulse, 
					S_STOP_ACQ_ENA, 
					S_start_readout, 
					S_write_fast_addr, 
					S_raise_TKI, 
					S_raise_TKI_2, 
					S_check_FLAG_A, 
					S_store_channel_data_A, 
					S_start_ADC_A,
					S_STORE_tdo_A, 
					S_wait_state1_A,
					S_UDP_HEADER,
					S_UDP_data,
					S_UDP_END_EVENT,
					S_done);

	SIGNAL state			: state_type;
	SIGNAL CNT 	   		: STD_LOGIC_VECTOR(15 downto 0);
	SIGNAL COUNT	  		: STD_LOGIC_VECTOR(15 downto 0);

	SIGNAL reg_ck			: STD_LOGIC;
	SIGNAL reg_ena			: STD_LOGIC; 
	SIGNAL reg_wen			: STD_LOGIC; 
	SIGNAL reg_tki			: STD_LOGIC;
	SIGNAL reg_stp			: STD_LOGIC; 
	SIGNAL A_SCK			: STD_LOGIC;
	SIGNAL A_CSB			: STD_LOGIC;

	  
	  
	SIGNAL spi_enabled	: STD_LOGIC;	
	SIGNAL spi_ck			: STD_LOGIC;	
	SIGNAL spi_wen			: STD_LOGIC;	
	SIGNAL spi_ena			: STD_LOGIC;	
	SIGNAL spi_diA			: STD_LOGIC;	
	  
	  
	  
	SIGNAL FLAGA			: STD_LOGIC;

	SIGNAL diA				: STD_LOGIC;
	SIGNAL diA_dly			: STD_LOGIC;
	SIGNAL CLK_CNT			: STD_LOGIC_VECTOR(7 downto 0);
	  
	SIGNAL HEADER   		: STD_LOGIC_VECTOR(15 downto 0);
	SIGNAL data_sel		: STD_LOGIC_VECTOR(3 downto 0);
	SIGNAL PDOA	   		: STD_LOGIC_VECTOR(15 downto 0);
	SIGNAL TDOA	   		: STD_LOGIC_VECTOR(15 downto 0);
	SIGNAL Start_ADC		: STD_LOGIC;
	SIGNAL STRT_ADC		: STD_LOGIC;
	SIGNAL USB_wrreq		: STD_LOGIC;
	SIGNAL TEST	   		: STD_LOGIC_VECTOR(15 downto 0);
	SIGNAL ADC_RDY			: STD_LOGIC;
	SIGNAL ADC_RDY_S		: STD_LOGIC;

	
	SIGNAL hs_vmm_wrreq		: STD_LOGIC;
	SIGNAL hs_vmm_pktend		: STD_LOGIC;	
	SIGNAL hs_vmm_data		: STD_LOGIC_VECTOR(7 downto 0);	
	
	SIGNAL hs_ADC_wrreq		: STD_LOGIC;
	SIGNAL hs_ADC_pktend		: STD_LOGIC;	
	SIGNAL hs_ADC_data		: STD_LOGIC_VECTOR(7 downto 0);	
	
	SIGNAL hs_test_wrreq		: STD_LOGIC;
	SIGNAL hs_test_pktend 	: STD_LOGIC;
	SIGNAL hs_test_data		: STD_LOGIC_VECTOR(7 downto 0);
	SIGNAL UDP_data			: STD_LOGIC_VECTOR(31 downto 0);
	
	SIGNAL trig_count			: STD_LOGIC_VECTOR(31 downto 0);
	SIGNAL trig_count_lat	: STD_LOGIC_VECTOR(31 downto 0);
   SIGNAL udp_cnt				: INTEGER RANGE 0 TO 31; 
   SIGNAL VALID_DATA			: STD_LOGIC; 
	SIGNAL hs_ff_wr_full_d	: STD_LOGIC; 
	SIGNAL trig_timeout_CNT	: STD_LOGIC_VECTOR(15 downto 0);
	SIGNAL Trigger_SYNC		: STD_LOGIC; 
	SIGNAL Trigger_LATCH		: STD_LOGIC; 
	SIGNAL Trigger_mode		: STD_LOGIC; 
	SIGNAL trig_rst			: STD_LOGIC; 
	SIGNAL ASIC_RST			: STD_LOGIC; 
	SIGNAL vmm_ena_RST		: STD_LOGIC;
	SIGNAL vmm_wen_RST		: STD_LOGIC;
	SIGNAL ASIC_RST_CNT		: INTEGER RANGE 0 TO 255;
	
	
	
	SIGNAL ADC_START_UDP		: STD_LOGIC;

	
	
	
begin





inst_ADC_PDO_A :  ADC_AD7276_OS
PORT MAP(clk   		=> ADC_CLK,
			rst      	=> rst,
			start   		=> STRT_ADC,
			OVER_SAMP	=> ADC_OVSAMP,
			ADC_SDO  	=> ADC_Pdo_A,
			ADC_SCK		=> A_SCK,
			ADC_CS		=> A_CSB,
			DATA_OUT		=> PDOA,
			RDY			=> ADC_RDY_S);

			
	
	
			
inst_ADC_TDO_A :  ADC_AD7276_OS
PORT MAP(clk   		=> ADC_CLK,
			rst      	=> rst,
			start   		=> STRT_ADC,
			OVER_SAMP	=> ADC_OVSAMP,
			ADC_SDO  	=> ADC_Tdo_A,
			ADC_SCK		=> open,
			ADC_CS		=> open,
			DATA_OUT		=> TDOA,
			RDY			=> open);


			
			
			
inst_VMM_SPI : VMM_SPI 
PORT MAP(			
		rst 	   		=>	rst,
		clk_1Mhz   		=> SPI_CLK,
		spi_START		=> spi_START,
		spi_address 	=> spi_address,
		spi_data			=> spi_data,
		spi_enabled		=> spi_enabled,
		vmm_ck			=> spi_ck,
		vmm_wen			=> spi_wen,
		vmm_ena			=> spi_ena,
		vmm_diA			=> spi_diA
);
			
	

	
	
lx9_test_inat : lx9_test  
	PORT MAP(
		clk      		=> clk,
		rst     			=> rst,
		start   			=> TEST_EN,
		speed				=> x"0000",
		PKT_DLY			=> x"00002000",
		hs_ffdata 		=> hs_test_data,
		hs_ff_PKT_END	=> hs_test_pktend,
		hs_ff_wrreq 	=> hs_test_wrreq,
		hs_ff_wr_full  => hs_ff_wr_full	
	);
		
		
ADC_UDP_TX_inst  : ADC_UDP_TX   
	PORT MAP(
		clk      			=> clk,
		rst     				=> rst,
		start   				=> TEST_EN,
		srs_header			=> srs_header,
		ADC1_DATA			=> PDOA,
		ADC2_DATA			=> TDOA,
		ADC_RDY				=> ADC_RDY_S,
		ADC_START_UDP		=> ADC_START_UDP,
		hs_ADC_wrreq		=> hs_ADC_wrreq,
		hs_ADC_pktend		=> hs_ADC_pktend,
		hs_ADC_data			=> hs_ADC_data,
		hs_ff_wr_full  	=> hs_ff_wr_full
);
			

		
		vmm_diA		<= spi_diA		when (spi_enabled = '1') else
							IO_vmm_diA;				
		vmm_ck		<= spi_ck		when (spi_enabled = '1') else
							IO_vmm_ck	when (IO_EN = '1') 		 else 
							reg_ck;
		vmm_ena		<= vmm_ena_RST			when (ASIC_RST = '1') 	  else
							not Trigger_LATCH when (Trigger_mode = '1') else
							spi_ena			   when (spi_enabled = '1')  else
							IO_vmm_ena			when (IO_EN = '1') 		  else 
							reg_ena;
		vmm_wen		<= vmm_wen_RST	when (ASIC_RST = '1')    else
							spi_wen		when (spi_enabled = '1') else
							IO_vmm_wen	when (IO_EN = '1')       else 
							reg_wen;
		vmm_tkiA		<= IO_vmm_tkiA	when (IO_EN = '1') else 
							reg_tki;
		vmm_stp		<= IO_vmm_stp	when (IO_EN = '1') else 
							reg_stp;
		ADC_SCK		<= IO_VMM_SCK	when (IO_EN = '1') else 
							A_SCK;
		ADC_CSB		<= IO_VMM_CSB	when (IO_EN = '1') else 
							A_CSB;


		USB_HS_CLK	<= clk;
	
		ADC_1_DAT	<=	PDOA;
		ADC_2_DAT	<=	TDOA;

		
		
		USB_wr_req	<= USB_wrreq;
		
		STRT_ADC	<=		ADC_START_UDP										when (DATA_MODE = b"10") 		else
							Start_ADC 											when (usb_ADC_TST  = b"00")	else 
							ADC_TST;
		USB_DATA	<=		TEST													when (usb_ADC_TST  = b"10") 	else
							HEADER							    				when (data_sel = x"0") 			else
							PDOA													when (data_sel = x"2") 			else
							TDOA													when (data_sel = x"3") 			else							
							HEADER(15 downto 6) & vmm_A    				when (data_sel = x"6") 			else
							x"0000";	
	
		hs_vmm_data		<=	X"00"							 		when (udp_cnt = 0) else
								trig_count_lat(23 downto 16)	when (udp_cnt = 1) else
								trig_count_lat(15 downto 8)	when (udp_cnt = 2) else
								trig_count_lat(7  downto 0) 	when (udp_cnt = 3) else
								X"00"									when (udp_cnt = 4) else
								X"4C"									when (udp_cnt = 5) else
								X"4E"									when (udp_cnt = 6) else
								X"42"							 		when (udp_cnt = 7) else
								srs_header(31 downto 24)		when (udp_cnt = 8) else
								srs_header(23 downto 16)		when (udp_cnt = 9) else
								srs_header(15 downto 8) 		when (udp_cnt = 10) else
								srs_header(7 downto 0)		  	when (udp_cnt = 11) else
								UDP_data(31 downto 24)			when (udp_cnt = 12) else
								UDP_data(23 downto 16)			when (udp_cnt = 13) else
								UDP_data(15 downto 8) 			when (udp_cnt = 14) else
								UDP_data(7 downto 0)		  		when (udp_cnt = 15) else
								x"FA"									when (udp_cnt = 16) else
								x"FA";
							
													
							
		hs_ff_wr_full_d	<= hs_ff_wr_full	when (DATA_MODE = b"01") else	'0';								
		
		hs_ff_wrreq		<= '0'			  when (DATA_MODE = b"00") else
								hs_vmm_wrreq  when (DATA_MODE = b"01") else
								hs_ADC_wrreq  when (DATA_MODE = b"10") else
								hs_test_wrreq when (DATA_MODE = b"11");

								
		hs_ff_PKT_END	<= '0'				when (DATA_MODE = b"00") else
								hs_vmm_pktend  when (DATA_MODE = b"01") else
								hs_ADC_pktend  when (DATA_MODE = b"10") else
								hs_test_pktend when (DATA_MODE = b"11");
										
								
		hs_ffdata		<= x"00"			 when (DATA_MODE = b"00") else
								hs_vmm_data  when (DATA_MODE = b"01") else
								hs_ADC_data  when (DATA_MODE = b"10") else
								hs_test_data when (DATA_MODE = b"11");
																	
	

	
 process(Trigger,rst,trig_cnt_rst)  
  begin
		if (rst = '1') or (trig_cnt_rst = '1') then
			trig_count <= x"00000000";
			trig_count_lat	<= x"00000000";
		elsif (Trigger'event  AND  Trigger = '1') then
				trig_count <= trig_count + 1;
				trig_count_lat	<= trig_count;
	  end if;
end process;	


	
 process(Trigger,rst,trig_rst)  
  begin
		if (rst = '1') or (trig_rst = '1') then
			Trigger_LATCH	<= '0';
		elsif (Trigger'event  AND  Trigger = '1') then
			Trigger_LATCH	<= '1';
	  end if;
end process;	
	
	
	
	
 process(clk,rst,ASIC_RESET)  
  begin
		if (rst = '1') then
			ASIC_RST			<= '0';
			vmm_ena_RST		<= '0';
			vmm_wen_RST		<= '0';
			ASIC_RST_CNT	<= 0;
		elsif (clk'event  AND  clk = '1') then
			if(ASIC_RESET = '0') then
				ASIC_RST			<= '0';
				vmm_ena_RST		<= '0';
				vmm_wen_RST		<= '0';
				ASIC_RST_CNT	<= 0;			
			else
				ASIC_RST_CNT	<= 	ASIC_RST_CNT +1;			
					case ASIC_RST_CNT is 
					when 1 =>	ASIC_RST			<= '1';
					when 20 =>	vmm_wen_RST		<= '1';
					when 40 =>	vmm_ena_RST		<= '1';
					when 60 =>	vmm_ena_RST		<= '0';
					when 80 =>	vmm_wen_RST		<= '0';
					when 90 =>	ASIC_RST			<= '0';
					when 100 =>	ASIC_RST_CNT	<= 95;
					when others =>
				end case;	
			end if;
	  end if;
end process;		
			
	
--- SYNC TO  50Mhz CLOCK ----	
	
 process(clk,rst)  
  begin
		if (rst = '1') then
			TEST <= x"0000";
		elsif (clk'event  AND  clk = '1') then

			FLAGA 			<= vmm_flagA or ADC_TST;
			diA				<= (IO_vmm_diA ) xor  M_PUL_EDG_SEL ;
			diA_dly			<= diA;
			TEST				<= TEST + 1;
			ADC_RDY			<= ADC_RDY_S;
			Trigger_SYNC	<= Trigger_LATCH;
	  end if;
end process;		
		

-- start main read out

  process(clk,rst) 
  begin

     if ((rst = '1') or (IO_EN = '1') or (spi_enabled = '1')) then
		HEADER			<= x"ffff";
		data_sel			<= x"0";
		reg_ck			<= '0';
		reg_ena			<= '0';
		reg_wen			<= '0';
		reg_tki			<= '0';
		reg_stp			<= '0';
		CNT				<= x"0000";
		COUNT				<= x"0000";
		trig_timeout_CNT <= x"0000";
		state 			<= S_IDLE;
		USB_wrreq		<= '0';
		Start_ADC		<= '0';
		hs_vmm_pktend	<= '0';
		hs_vmm_wrreq	<= '0';
		udp_cnt			<= 0;
		VALID_DATA		<= '0';
		CLK_CNT			<= x"00";
		trig_rst			<= '1';		
		Trigger_mode	<= '0';
		busy				<= '0';
		elsif (clk'event AND clk = '1') then
		CASE state IS
		when S_IDLE =>	
			HEADER		<= x"ffff";
			data_sel		<= x"0";
			reg_ck		<= '0';
			busy			<= '0';
			reg_ena		<= '0';
			reg_wen		<= '0';
			reg_tki		<= '0';
			reg_stp		<= '0';
			CNT			<= x"0000";
			COUNT			<= x"0000";
			trig_timeout_CNT <= x"0000";			
			USB_wrreq	<= '0';
			Start_ADC	<= '0';
			hs_vmm_pktend	<= '0';
			hs_vmm_wrreq	<= '0';
			udp_cnt			<= 0;
			CLK_CNT			<= x"00";	
			VALID_DATA		<= '0';		
			trig_rst		<= '1';			
			Trigger_mode	<= '0';
			if (ACQ_enable  = '1')  then  		-- start acq with a system register	
					state 		<= S_delay_start;
         end if;					
		when S_delay_start	=>						-- add some delay if needed 
			COUNT			<= COUNT +1;
			if (Start_DLY <=  COUNT	)  then  			
				state 		<= S_acq_reset_ENA ;
				reg_wen		<= '1';
				COUNT			<= x"0000";
         end if;				
		when S_acq_reset_ENA =>		---ASIC  FIX --  ADD RESET BEFORE READOUT
			CNT			<= CNT + 1;
			trig_timeout_CNT <= x"0000";
			trig_rst		<= '1';
			if(CNT = 10) then
				reg_wen		<= '0';
			elsif(CNT = 20) then
				reg_ena		<= '1';
			elsif(CNT = 30) then
				reg_ena		<= '0';	
			elsif(CNT = 40) then
				reg_wen		<= '1';
			elsif(CNT = 60) then
				reg_wen		<= '0';			
			elsif (CNT >= 90) then
				if(M_trigger = '1') then		
					CNT			<= x"0000";
					reg_ena		<= '0';	
					Trigger_mode<= '1';
					state 		<= S_wait_trigger;
				else
					reg_ena		<= '1';
					state 		<= S_wait_for_flag_or_tpulse;
				end if;
			end if;
			
			
		when S_fast_reset_ENA =>		---ASIC  FIX --  ADD RESET BEFORE READOUT
			CNT			<= CNT + 1;
			trig_timeout_CNT <= x"0000";
			trig_rst		<= '1';
			if(CNT = 2) then	
				reg_wen		<= '1';
			elsif(CNT = 5) then
				reg_wen		<= '0';			
			elsif (CNT >= 8) then
				CNT			<= x"0000";
				reg_ena		<= '0';	
				Trigger_mode<= '1';
				state 		<= S_wait_trigger;
			end if;					
		when	S_wait_trigger =>
			trig_rst		<= '0';
			if(FLAGA = '1') then
					trig_timeout_CNT	<= trig_timeout_CNT + 1;
			end if;
			if(trig_timeout_CNT >= trig_timeout) then
					state 		<= S_fast_reset_ENA;
					reg_ena		<= '0';
					CNT			<= x"0000"; 
					trig_rst		<= '1';
					Trigger_mode<= '0';
			end if;
			if(Trigger_SYNC = '1')  then
				if(FLAGA = '1') then
					reg_ena		<= '0';
					Trigger_mode<= '0';
					trig_rst		<= '1';
					state 		<= S_start_readout;
				else
									trig_rst		<= '1';
				

				end if;
			end if;		
		when S_wait_for_flag_or_tpulse =>		-- allow for 2 timing modes flag to enable or Tpulse to enable
			CNT	<= x"0000";
			if(FLAGA = '1') then
				state 		<= S_STOP_ACQ_ENA;
			end if;
			if ((diA = '1') and (diA_dly = '0') and (M_PUL_TO_EN = '1')) then
				state 		<= S_STOP_ACQ_ENA;
		   end if;
		when S_STOP_ACQ_ENA =>					-- lower enable
			CNT		<= CNT + 1;	
			if((ENABLE_TIME <= CNT) and (FLAGA = '1')) then 
					reg_ena		<= '0';
					state 		<= S_start_readout;
			end if;
		when S_start_readout =>			-- send start of event  to USB
				busy			<= '1';
				HEADER		<= x"ffff";
				data_sel		<= x"0";
				USB_wrreq	<= '1';
				state 		<= S_write_fast_addr;
		when S_write_fast_addr =>		-- send fast address data to usb
				data_sel		<= x"1";
				USB_wrreq	<= '1';	
				udp_cnt		<= 0;
				if( hs_ff_wr_full_d = '0') then
					state 		<= S_UDP_HEADER;
					hs_vmm_wrreq<= '1';
				end if;
		when S_UDP_HEADER =>			
				USB_wrreq		<= '0';					
				hs_vmm_wrreq	<= '0';
				if( hs_ff_wr_full_d = '0') then
					udp_cnt			<= udp_cnt + 1;
					hs_vmm_wrreq	<= '1';
					if(udp_cnt = 10) then
							state 		<= S_raise_TKI;
					end if;
				end if;
		when S_raise_TKI =>				-- send token into first chip
				hs_vmm_wrreq<= '0';
				udp_cnt		<= 12;
				USB_wrreq	<= '0';
				VALID_DATA	<= '0';
				data_sel		<= x"6";
				reg_tki		<= '1';
				reg_ck		<= '1';
				state 		<= S_raise_TKI_2;
		when S_raise_TKI_2 =>
				reg_tki		<= '1';
				CLK_CNT		<= x"00";
				state 		<= S_check_FLAG_A;
		when S_check_FLAG_A =>		-- check for data in chip a or go to next chip
				CLK_CNT		<= CLK_CNT + 1;
				reg_ck		<= '1';
				udp_cnt		<= 12;
				data_sel		<= x"6";
				USB_wrreq	<= '0';
				COUNT			<= x"0000";
				if(CLK_CNT > 68) then   -- flag is stuck go to next chip				
					if( hs_ff_wr_full_d = '0') then
						state 		<= S_UDP_END_EVENT;
						hs_vmm_wrreq<= '1';
					end if;
					CLK_CNT		<= x"00";
				elsif(FLAGA = '1') then			-- Store address and start channel readout to be sent to usb
					state 		<= S_store_channel_data_A;
					HEADER		<=	x"A0" & b"00" & vmm_A;
				else
					if( hs_ff_wr_full_d = '0') then
						state 		<= S_UDP_END_EVENT;
						hs_vmm_wrreq<= '1';
					end if;
				end if;
		when S_store_channel_data_A =>		-- send channel address to USB
				USB_wrreq	<= '1';
				if(VALID_DATA = '1') then
					if( hs_ff_wr_full_d = '0') then
						state 		<= S_UDP_data;
						udp_cnt		<= 12;
						hs_vmm_wrreq<= '1';
					end if;
				else
					state 		<=  S_start_ADC_A;
				end if;
		when S_UDP_data =>		
				VALID_DATA		<= '0';	
				USB_wrreq		<= '0';					
				hs_vmm_wrreq	<= '0';
				data_sel			<= x"2";
				reg_ck			<= '1';
				if(hs_ff_wr_full_d = '0') then
					udp_cnt			<= udp_cnt + 1;
					hs_vmm_wrreq	<= '1';
					if(udp_cnt = 14) then
							state 		<=  S_start_ADC_A;
					end if;
				end if;				
		when S_start_ADC_A =>				-- start ADC conversion  for PDO and TDO
				VALID_DATA	<= '1';
				udp_cnt		<= 12;
				hs_vmm_wrreq<= '0';
				USB_wrreq	<= '0';
				data_sel		<= x"2";
				COUNT <= COUNT +1;
				if(COUNT >= ADC_start_dly) then
					Start_ADC	<= '1';
				end if;
				if(COUNT >= ADC_start_dly +10) then  -- wait for adc to finish conversion
					Start_ADC	<= '0';
					COUNT	<= COUNT;
					if(ADC_RDY = '1') then		-- ADC DONE 
						USB_wrreq	<= '1';
						UDP_data		<=	vmm_A & PDOA(12 downto 0) & TDOA(12 downto 0) ;
						reg_ck		<= '0';  
						state 		<= S_STORE_tdo_A;
					end if;
				end if;
		when S_STORE_tdo_A =>		-- SEND PDO DATA TO USB
				data_sel		<= x"3";
				USB_wrreq	<= '1';
				reg_ck		<= '0';
				reg_tki		<= '0';
				COUNT			<= x"0000";
				state 		<= S_wait_state1_A;
		when S_wait_state1_A =>			-- SEND TDO DATA  TO USB  , ADD DELAY for flag to settle
				USB_wrreq	<= '0';
				udp_cnt		<= 12;
				reg_ck		<= '1';
				COUNT <= COUNT +1 ;
				USB_wrreq	<= '0';
				if(COUNT >= 6) then
					state 		<= S_check_FLAG_A;			
					COUNT			<= x"0000";
				end if;		
		when S_UDP_END_EVENT =>		
				USB_wrreq		<= '0';					
				hs_vmm_wrreq	<= '0';
				hs_vmm_pktend	<= '0';
				if( hs_ff_wr_full_d = '0') then
					udp_cnt		<= udp_cnt + 1;
					hs_vmm_wrreq	<= '1';
					if(udp_cnt = 14) then
						hs_vmm_pktend	<= '1';
					else
						hs_vmm_pktend	<= '0';
					end if;
					if(udp_cnt = 18) then			
						state 			<= S_done;
						hs_vmm_pktend	<= '1';
					end if;
							
				end if;				
		
		when S_done =>		---ASIC  FIX --  ADD MORE CLOCKS AT END OF READOUT
				USB_wrreq		<= '0';					
				hs_vmm_wrreq	<= '0';
				hs_vmm_pktend	<= '0';
		
				COUNT <= COUNT +1;
				if( COUNT = 1)   then
						reg_ck		<= '1';
				elsif( COUNT = 5)   then
						reg_ck		<= '0';
				elsif( COUNT = 10)   then
						reg_ck		<= '1';
				elsif( COUNT = 15)   then
						reg_ck		<= '0';
				elsif( COUNT = 20)   then
						reg_ck		<= '1';
				elsif( COUNT = 25)   then
						reg_ck		<= '0';
				elsif( COUNT = 30)   then
						reg_ck		<= '1';
				elsif( COUNT = 35)   then
						reg_ck		<= '0';
				elsif( COUNT = 40)   then
						reg_ck		<= '1';
				elsif( COUNT = 50)   then
						reg_ck		<= '0';
				elsif( COUNT = 55)   then
						reg_ck		<= '1';
				elsif( COUNT = 60)   then
						reg_ck		<= '0';
				elsif( COUNT = 65)   then
						reg_ck		<= '1';
				elsif( COUNT = 70)   then
						reg_ck		<= '0';						
				elsif( COUNT = 75)   then
						reg_ck		<= '1';
				elsif( COUNT = 80)   then
						reg_ck		<= '0';					
				end if;
	
				if( COUNT > 90) then
					busy			<= '0';
					state 		<= S_IDLE;
				end if;
		when others =>	
			state <= S_idle;		
      end case;  
     end if;
end process;
	
	
	
END behavior;