--*********************************************************
--* FILE  : MIOS_USB.VHD
--* Author: Jack Fried
--*
--* Last Modified: 5/19/2005
--*  
--* Description: interface to the cypress USB FIFO's
--*		 		               
--*
--*
--*
--*
--*
--*
--*********************************************************

LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;


--  Entity Declaration

ENTITY reg_cntl IS

	PORT
	(
		clk         : IN STD_LOGIC;				-- clock
		rst         : IN STD_LOGIC;				-- state machine reset
		FIFO_data	: IN  STD_LOGIC_VECTOR(15 downto 0);	-- FIFO control data out
		FIFO_EMPTY	: IN STD_LOGIC;			     -- FIFO2  full flag
		FIFO_rdreq	: OUT STD_LOGIC;			-- FIFO1  read  request
		FIFO_rdclk	: OUT STD_LOGIC;			-- FIFO2  write request
		address     : OUT STD_LOGIC_VECTOR(15 downto 0);
		data		: OUT STD_LOGIC_VECTOR(15 downto 0);
		USB_strb	: OUT STD_LOGIC;	
		USB_dir	: OUT STD_LOGIC	
	);
	-- {{ALTERA_IO_END}} DO NOT REMOVE THIS LINE!
	
END reg_cntl;


ARCHITECTURE behavior OF reg_cntl IS

  type state_type is (S_idle,S_rdata,S_rdaddr,S_wdata,S_wr_strb,S_dly1,S_dly2);
  signal state     : state_type;
  Signal tmp_addr  : std_logic_vector(15 downto 0);	

begin

		FIFO_rdclk	<=  clk;
  process(clk,rst) 
  begin
     if (rst = '1') then
		state       <= S_idle;
		FIFO_rdreq  <= '0';
		USB_strb    <= '0';
		USB_dir     <= '0';
     elsif (clk'event AND clk = '0') then
        CASE state IS
     when S_IDLE =>	
		FIFO_rdreq  <= '0';
		USB_strb    <= '0';
		USB_dir     <= '0';
		if (FIFO_EMPTY = '0') then 
			state <= s_dly1;
		else
			state <= S_idle;
		end if;
	 when S_dly1 =>	
			state <= s_dly2;
	 when S_dly2 =>
			state <= S_rdata;
	 when S_rdata =>
					FIFO_rdreq  <= '1';
					state <= S_rdaddr;				
	 when S_rdaddr =>
					FIFO_rdreq  <= '1';
	    			data		<=  FIFO_data;	
					tmp_addr 	<=  FIFO_data;	
					state 		<=  S_wdata;						
	 when S_wdata =>
					FIFO_rdreq  <= '0';	
					if(FIFO_data = x"ffff") then 
						address	  <=  tmp_addr;
						USB_dir   <= '1';
					else
						address	   <=  FIFO_data;
						USB_dir    <= '0';
					end if;
					state <= S_wr_strb;				
	when S_wr_strb =>				
					USB_strb    <= '1';		
					state 		<= S_idle;					
						
     end case;  
end if;
end process;
END behavior;
