
LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;


ENTITY USB_TOP IS 
	port
	(
	
		USB_IO_CLK 		:  IN  STD_LOGIC;
		reset 			:  IN  STD_LOGIC;	
		c_ctl0_flaga 	:  IN  STD_LOGIC;
		c_ctl1_flagb 	:  IN  STD_LOGIC;
		c_ctl2_flagac 	:  IN  STD_LOGIC;
		c_clkout 		:  IN  STD_LOGIC;	
		c_int0 			:  IN  STD_LOGIC;	
		c_int1 			:  IN  STD_LOGIC;		
		hs_ffdata 		:  IN  STD_LOGIC_VECTOR(15 downto 0);
		hs_ff_wrreq 	:  IN  STD_LOGIC;
		hs_ffclk 		:  IN  STD_LOGIC;	
		data_out 		:  IN  STD_LOGIC_VECTOR(15 downto 0);
		c_ifclk			:  OUT  STD_LOGIC;			
		c_fd 			:  INOUT  STD_LOGIC_VECTOR(15 downto 0);
		c_pa2_sloe 		:  OUT  STD_LOGIC;
		c_rdy0_slrd 	:  OUT  STD_LOGIC;
		c_rdy1_slwr 	:  OUT  STD_LOGIC;		
		c_fifoadr0 		:  OUT  STD_LOGIC;
		c_fifoadr1 		:  OUT  STD_LOGIC;
		c_pktend 		:  OUT  STD_LOGIC;
		c_WU1			:  OUT  STD_LOGIC;
		c_WU2			:  OUT  STD_LOGIC;
		hs_ff_wr_full 	:  OUT  STD_LOGIC;
		address 		:  OUT  STD_LOGIC_VECTOR(15 downto 0);
		data 			:  OUT  STD_LOGIC_VECTOR(15 downto 0);	
		USB_strb 		:  OUT  STD_LOGIC;
		USB_dir 		:  OUT  STD_LOGIC;
		sys_clk			:  in  STD_LOGIC

	);
END USB_TOP;

ARCHITECTURE behavior OF USB_TOP IS 


component reg_cntl
	PORT(clk 			: IN STD_LOGIC;
		 rst 				: IN STD_LOGIC;
		 FIFO_EMPTY 	: IN STD_LOGIC;
		 FIFO_data		: IN STD_LOGIC_VECTOR(15 downto 0);
		 FIFO_rdreq 	: OUT STD_LOGIC;
		 FIFO_rdclk 	: OUT STD_LOGIC;
		 USB_strb 		: OUT STD_LOGIC;
		 USB_dir 		: OUT STD_LOGIC;
		 address 		: OUT STD_LOGIC_VECTOR(15 downto 0);
		 data 			: OUT STD_LOGIC_VECTOR(15 downto 0)
	);
end component;

component usb_hs_fifo
	PORT(wrreq 			: IN STD_LOGIC;
		 wrclk 			: IN STD_LOGIC;
		 rdreq 			: IN STD_LOGIC;
		 rdclk 			: IN STD_LOGIC;
		 aclr 			: IN STD_LOGIC;
		 data 			: IN STD_LOGIC_VECTOR(15 downto 0);
		 rdempty 		: OUT STD_LOGIC;
		 q 				: OUT STD_LOGIC_VECTOR(15 downto 0);
		 wrusedw 		: OUT STD_LOGIC_VECTOR(12 downto 0)
	);
end component;

component usb_cntl
	PORT(clk 			: IN STD_LOGIC;
		 rst 			: IN STD_LOGIC;
		 c_EP2FF 		: IN STD_LOGIC;
		 c_EP6EF 		: IN STD_LOGIC;
		 c_EP8FF 		: IN STD_LOGIC;
		 F1_empty 		: IN STD_LOGIC;
		 F2_full 		: IN STD_LOGIC;
		 F3_empty 		: IN STD_LOGIC;
		 c_FD 			: INOUT STD_LOGIC_VECTOR(15 downto 0);
		 F1_d 			: IN STD_LOGIC_VECTOR(15 downto 0);
		 F3_d 			: IN STD_LOGIC_VECTOR(15 downto 0);
		 c_SLOE 		: OUT STD_LOGIC;
		 c_SLRD 		: OUT STD_LOGIC;
		 c_SLWR 		: OUT STD_LOGIC;
		 c_FADDR0 		: OUT STD_LOGIC;
		 c_FADDR1 		: OUT STD_LOGIC;
		 c_PKTEND		: OUT STD_LOGIC;
		 F1_rclk 		: OUT STD_LOGIC;
		 F2_wclk 		: OUT STD_LOGIC;
		 F3_rclk 		: OUT STD_LOGIC;
		 F1_rdreq 		: OUT STD_LOGIC;
		 F2_wrreq 		: OUT STD_LOGIC;
		 F3_rdreq 		: OUT STD_LOGIC;
		 F2_d 			: OUT STD_LOGIC_VECTOR(15 downto 0)
	);
end component;

component usb_ls_fifo
	PORT(wrreq 			 : IN STD_LOGIC;
		 wrclk			 : IN STD_LOGIC;
		 rdreq 		 	: IN STD_LOGIC;
		 rdclk 		 	: IN STD_LOGIC;
		 aclr		 	: IN STD_LOGIC;
		 data		 	: IN STD_LOGIC_VECTOR(15 downto 0);
		 wrfull		 	: OUT STD_LOGIC;
		 rdempty 	 	: OUT STD_LOGIC;
		 q 			 	: OUT STD_LOGIC_VECTOR(15 downto 0)
	);
end component;

signal	f1_empty		:  STD_LOGIC;
signal	f1_q 			:  STD_LOGIC_VECTOR(15 downto 0);
signal	f1_rclk 		:  STD_LOGIC;
signal	f1_rdreq 		:  STD_LOGIC;
signal	f2_d 			:  STD_LOGIC_VECTOR(15 downto 0);
signal	f2_empty 		:  STD_LOGIC;
signal	f2_full 		:  STD_LOGIC;
signal	f2_q 			:  STD_LOGIC_VECTOR(15 downto 0);
signal	f2_rclk 		:  STD_LOGIC;
signal	f2_rdreq 		:  STD_LOGIC;
signal	f2_wclk 		:  STD_LOGIC;
signal	f2_wrreq 		:  STD_LOGIC;
signal	f3_empty 		:  STD_LOGIC;
signal	f3_q 			:  STD_LOGIC_VECTOR(15 downto 0);
signal	f3_rclk 		:  STD_LOGIC;
signal	f3_rdreq 		:  STD_LOGIC;
signal	USB_dir_s 		:  STD_LOGIC;
signal	USB_strb_s 		:  STD_LOGIC;
signal   HS_FIFO_D_COUNT :  STD_LOGIC_VECTOR(12 downto 0);
signal	USB_LS_F_wrreq 	:  STD_LOGIC;


signal	hs_ffdata_r 	:  STD_LOGIC_VECTOR(15 downto 0);
signal	hs_ff_wrreq_r 	:  STD_LOGIC;



BEGIN 
	c_ifclk				<= USB_IO_CLK;
	USB_strb 			<= USB_strb_s;
	USB_dir 				<= USB_dir_s;
	USB_LS_F_wrreq 	<= USB_strb_s AND USB_dir_s;
	hs_ff_wr_full 		<=  '1'	when ( HS_FIFO_D_COUNT > x"1fd0") else    '0';
	c_WU1					<= '1';
	c_WU2					<= '1';
	
	
	
  process(hs_ffclk) 
  begin
	if (hs_ffclk'event AND hs_ffclk = '1') then
		hs_ffdata_r 		<= hs_ffdata;
		hs_ff_wrreq_r 		<= hs_ff_wrreq;
	end if;
  end process;
	
	
inst1 : reg_cntl
PORT MAP(
		 clk		=> sys_clk,
		 rst 		=> reset,
		 FIFO_EMPTY => f2_empty,
		 FIFO_data 	=> f2_q,
		 FIFO_rdreq => f2_rdreq,
		 FIFO_rdclk => f2_rclk,
		 USB_strb 	=> USB_strb_s,
		 USB_dir 	=> USB_dir_s,
		 address 	=> address,
		 data 		=> data);

inst2 : usb_hs_fifo
PORT MAP(wrreq 		=> hs_ff_wrreq_r,
		 wrclk 		=> hs_ffclk,
		 rdreq 		=> f1_rdreq,
		 rdclk 		=> f1_rclk,
		 aclr 		=> reset,
		 data 		=> hs_ffdata_r,
		 rdempty 	=> f1_empty,
		 q 			=> f1_q,
		 wrusedw 	=> HS_FIFO_D_COUNT);

inst3 : usb_cntl
PORT MAP(clk 		=> USB_IO_CLK,
		 rst 		=> reset,
		 c_EP2FF 	=> c_ctl0_flaga,
		 c_EP6EF 	=> c_ctl1_flagb,
		 c_EP8FF 	=> c_ctl2_flagac,
		 F1_empty 	=> f1_empty,
		 F2_full 	=> f2_full,
		 F3_empty 	=> f3_empty,
		 c_FD 		=> c_fd,
		 F1_d 		=> f1_q,
		 F3_d 		=> f3_q,
		 c_SLOE 	=> c_pa2_sloe,
		 c_SLRD 	=> c_rdy0_slrd,
		 c_SLWR 	=> c_rdy1_slwr,
		 c_FADDR0 	=> c_fifoadr0,
		 c_FADDR1 	=> c_fifoadr1,
		 c_PKTEND 	=> c_pktend,
		 F1_rclk 	=> f1_rclk,
		 F2_wclk 	=> f2_wclk,
		 F3_rclk 	=> f3_rclk,
		 F1_rdreq 	=> f1_rdreq,
		 F2_wrreq 	=> f2_wrreq,
		 F3_rdreq 	=> f3_rdreq,
		 F2_d 		=> f2_d);

inst4 : usb_ls_fifo
PORT MAP(wrreq 		=> f2_wrreq,
		 wrclk 		=> f2_wclk,
		 rdreq 		=> f2_rdreq,
		 rdclk 		=> f2_rclk,
		 aclr		=> reset,
		 data 		=> f2_d,
		 wrfull 	=> f2_full,
		 rdempty 	=> f2_empty,
		 q 			=> f2_q);

inst5 : usb_ls_fifo
PORT MAP(wrreq 		=> USB_LS_F_wrreq,
		 wrclk 		=> f2_rclk,
		 rdreq 		=> f3_rdreq,
		 rdclk 		=> f3_rclk,
		 aclr 		=> reset,
		 data 		=> data_out,
		 rdempty 	=> f3_empty,
		 q 			=> f3_q);
		
END; 