--*********************************************************
--* FILE  : VMM2_TST_PULSE.VHD
--* Author: Jack Fried
--*
--* Last Modified: 09/16/2014
--*  
--* Description: VMM2_TST_PULSE
--*		 		               
--*
--*
--*
--*
--*
--*
--*********************************************************

LIBRARY ieee;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;


--  Entity Declaration

ENTITY VMM2_TST_PULSE IS

	PORT
	(
		sys_rst     	: IN STD_LOGIC;			
		clk_40Mhz    	: IN STD_LOGIC;				-- clock
		TP_ENABLE		: IN STD_LOGIC;
		TP_PERIOD		: IN STD_LOGIC_VECTOR(31 downto 0);		--  0 = 25ns 1 = 50ns ......
		TP_WIDTH			: IN STD_LOGIC_VECTOR(15 downto 0);		--  0 = 25ns 1 = 50ns ......
		TP_P_TRAIN		: IN STD_LOGIC_VECTOR(7 downto 0);		--  0 = 1 , 1 = 1, 2 = 2 ......
		TST_PULSE		: OUT STD_LOGIC
	);
	

	END VMM2_TST_PULSE;

ARCHITECTURE behavior OF VMM2_TST_PULSE IS



	
	type state_type is (
						S_IDLE,
						S_GEN_TP_H,
						S_GEN_TP_L);

	SIGNAL state			: state_type;


 signal TP_PRD_CNT		: STD_LOGIC_VECTOR(31 downto 0);
 signal TP_WIDTH_CNT		: STD_LOGIC_VECTOR(15 downto 0);
 signal TP_TRN_CNT		: STD_LOGIC_VECTOR(7 downto 0);
 
 
begin
		 	 
				 
 process(clk_40Mhz,sys_rst) 
begin
	if  (sys_rst = '1') then
		TP_PRD_CNT		<= (others => '0');
		TP_WIDTH_CNT	<=	(others => '0');
		TP_TRN_CNT		<= x"01";		
		TST_PULSE		<= '0';
		state 			<= S_IDLE;
	elsif clk_40Mhz'event and clk_40Mhz = '1' then
		CASE state IS
		when S_IDLE =>		
				TST_PULSE		<= '0';
				TP_WIDTH_CNT	<=	(others => '0');
				TP_TRN_CNT		<= x"01";			
				if(TP_ENABLE = '1') then
					TP_PRD_CNT	<= TP_PRD_CNT + 1;
					if(TP_PRD_CNT >= TP_PERIOD) then
						TP_PRD_CNT		<= (others => '0');
						state 			<= S_GEN_TP_H;
					end if;
				else
					TP_PRD_CNT		<= (others => '0');
				end if;
		WHEN S_GEN_TP_H =>
				TST_PULSE		<= '1';
				TP_WIDTH_CNT <= TP_WIDTH_CNT + 1;
				if(TP_WIDTH_CNT > TP_WIDTH) then
					TP_TRN_CNT		<= TP_TRN_CNT + 1;
					state 			<= S_GEN_TP_L;
					TP_WIDTH_CNT	<=	(others => '0');
					if( TP_TRN_CNT >= TP_P_TRAIN) then
						state 			<= S_IDLE;
						TP_PRD_CNT		<= (others => '0');	
					end if;
				end if;
		WHEN S_GEN_TP_L =>
				TST_PULSE		<= '0';
				TP_WIDTH_CNT <= TP_WIDTH_CNT + 1;
				if(TP_WIDTH_CNT > TP_WIDTH) then
					state 			<= S_GEN_TP_H;
					TP_WIDTH_CNT	<=	(others => '0');
				end if;
		when others =>	
			state <= S_idle;		
      end case; 		
	end if;
end process;



END behavior;

	
	